use std::{env, thread, time};

mod wayland_protocol;
use wayland_protocol::init_wayland;
use wayland_protocol::TouchDown;

mod touch_pass;
use touch_pass::TouchState;

// The main function of our program
fn main() {
    let passcode = if let Ok(passcode) = env::var("PEANUTBUTTER_PASSCODE") {
        Some(passcode)
    } else {
        None
    };
    let result = init_wayland();
    let mut way_state = result.0;
    let mut event_queue = result.1;
    let wait_between_lock_unlock = time::Duration::from_secs(2);

    way_state.lock();
    way_state.round();
    let mut previous_down: Option<TouchDown> = None;
    let mut touch_state = TouchState::new(
        way_state.width.unwrap(),
        way_state.height.unwrap(),
        passcode.as_deref(),
    );
    loop {
        _ = event_queue.blocking_dispatch(&mut way_state);
        if let Some(last_down) = way_state.last_down.as_ref() {
            if previous_down.as_ref().is_none()
                || previous_down.as_ref().unwrap().serial != last_down.serial
            {
                previous_down = Some(*last_down);
                if touch_state.add_touch(last_down.x, last_down.y) {
                    way_state.flip()
                }
            }
        }
        if touch_state.is_pass_valid() {
            break;
        }
    }
    way_state.unlock();
    thread::sleep(wait_between_lock_unlock);
}
